@@ -891,9 +891,8 @@ static void thread_func(void *userdata) {
 
 /*        pa_log("loop");    */
 
-        if (u->sink && PA_SINK_IS_OPENED(u->sink->thread_info.state))
-            if (u->sink->thread_info.rewind_requested)
-                pa_sink_process_rewind(u->sink, 0);
+        if (PA_UNLIKELY(u->sink && u->sink->thread_info.rewind_requested))
+            pa_sink_process_rewind(u->sink, 0);
 
         /* Render some data and write it to the dsp */
 
